<?php
namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Geral\Util\Environment;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;


class Fai00800CalculoDaApuracaoGestaoDireta extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $environment;
	protected $crudService;
	protected $parameterBag;
	protected $dataSourceName;
	protected $parameterBagColumns;
	protected $dataSourceManager;
	protected $novoCodigo;
	protected $entityManager;

	public function __construct($crudService, $parameterBag, $environment, $dataSourceManager, $novoCodigo, $entityManager) {

		$this->crudService       = $crudService;
		$this->parameterBag      = $parameterBag;
		$this->environment       = $environment;
		$this->dataSourceManager = $dataSourceManager;
		$this->novoCodigo        = $novoCodigo;
		$this->entityManager     = $entityManager;
	}

	public function filtrocalcapuracao(DTO\Request\Filter $request, DTO\Response $response) {

		$filterCriteria = $request->getFilterCriteria();
		$this->dataSourceName = "fai00800_calcapurgestdir";
		$this->parameterBagColumns = array ("CDCLIENTE", "P_CDFILIAL","P_CDOPERADOR");
		parent::find($request, $response);
	}

	public function verificatabelacorrecao(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->addCondition('CDFILIAL', "=", $this->environment->getCdFilial());
		$this->dataSourceName = "fai00800_vertabcorrecao";
		$this->parameterBagColumns = array ("CDFILIAL","NRORG");
		parent::find($request, $response);
	}

	public function verificalocal(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$this->dataSourceName = "fai00800_utilizalocal";
		$this->parameterBagColumns = array ("CDFILIAL");
		parent::find($request, $response);
	}

	private function getNrApuracFat($rows) {

		$rowsToSave = array();
		foreach ($rows as $row) {
			$conditions = array(
				array(
					"columnName" => "CDCLIENTE",
					"operator" => "=",
					"value" => $row['CDCLIENTE']
				),
				array(
					"columnName" => "CDFILIAPUR",
					"operator" => "=",
					"value" => $row['CDFILIAPUR']
				)
			);
			$dataSet = $this->crudService->getFilteredDataSet("fai00800_ultima_apuracao_cliente", $conditions);
			$ultimaApuracao = $dataSet->getRows();
			$ultimaApuracao = count($ultimaApuracao) > 0 ? $ultimaApuracao[0] : null;
			if (!empty($ultimaApuracao) && $ultimaApuracao['IDBLOQUEADO'] != 'S') {
				$row["CDFILIAL"] = $this->environment->getCdFilial();
				$cdcontador = "APURACFATUR".$row["CDFILIAL"];
				$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
				$rowsToSave[] = array(
				"CDFILIAL"    => $row["CDFILIAL"],
				"NRAPURACFAT" => $row['NRAPURACFAT'],
				"CDCLIENTE"   => $row['CDCLIENTE'],
				"DTINIAPURAC" => $row['DTINIAPUR'],
				"DTFIMAPURAC" => $row['DTFINAPUR'],
				"CDFILIAPUR"  => $row["CDFILIAPUR"],
				"CDTIPOFATUCT"=> '001',
				"__is_new"    => true
				);
			} else {
				if (!empty($ultimaApuracao) && $ultimaApuracao['IDBLOQUEADO'] == 'S') {
					$ultimaDtiniapur = \DateTime::createFromFormat('!d/m/Y', substr($ultimaApuracao["DTINIAPURAC"], 0, 10));
					$ultimaDtfimapur = \DateTime::createFromFormat('!d/m/Y', substr($ultimaApuracao["DTFIMAPURAC"], 0, 10));
					$dtiniapur = \DateTime::createFromFormat('!d/m/Y', substr($row["DTINIAPUR"], 0, 10));
					$dtfimapur = \DateTime::createFromFormat('!d/m/Y', substr($row["DTFINAPUR"], 0, 10));
					if (($dtiniapur >= $ultimaDtiniapur && $dtiniapur <= $ultimaDtfimapur) ||
					($dtfimapur >= $ultimaDtiniapur && $dtfimapur <= $ultimaDtfimapur)) {
						throw new \Exception("Já existe nota emitida para o período apurado.", 1);
					} else {
						$row["CDFILIAL"] = $this->environment->getCdFilial();
						$cdcontador = "APURACFATUR".$row["CDFILIAL"];
						$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
						$rowsToSave[] = array(
							"CDFILIAL"    => $row["CDFILIAL"],
							"NRAPURACFAT" => $row['NRAPURACFAT'],
							"CDCLIENTE"   => $row['CDCLIENTE'],
							"DTINIAPURAC" => $row['DTINIAPUR'],
							"DTFIMAPURAC" => $row['DTFINAPUR'],
							"CDFILIAPUR"  => $row["CDFILIAPUR"],
							"CDTIPOFATUCT"=> '001',
							"__is_new"    => true
						);
					}
				} else {
					$row["CDFILIAL"] = $this->environment->getCdFilial();
					$cdcontador = "APURACFATUR".$row["CDFILIAL"];
					$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
					$rowsToSave[] = array(
						"CDFILIAL"    => $row["CDFILIAL"],
						"NRAPURACFAT" => $row['NRAPURACFAT'],
						"CDCLIENTE"   => $row['CDCLIENTE'],
						"DTINIAPURAC" => $row['DTINIAPUR'],
						"DTFIMAPURAC" => $row['DTFINAPUR'],
						"CDFILIAPUR"  => $row["CDFILIAPUR"],
						"CDTIPOFATUCT"=> '001',
						"__is_new"    => true
					);
				}
			}
		}
		return $rowsToSave;
	}
	
	private function getNrApuracFatForRow($row) {
		$rowToSave = $row;
		
		$conditions = array(
			array(
				"columnName" => "CDCLIENTE",
				"operator" => "=",
				"value" => $row['CDCLIENTE']
			),
			array(
				"columnName" => "CDFILIAPUR",
				"operator" => "=",
				"value" => $row['CDFILIAPUR']
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet("fai00800_ultima_apuracao_cliente", $conditions);
		$ultimaApuracao = $dataSet->getRows();
		$ultimaApuracao = count($ultimaApuracao) > 0 ? $ultimaApuracao[0] : null;
		if (!empty($ultimaApuracao) && $ultimaApuracao['IDBLOQUEADO'] != 'S') {
			$row["CDFILIAL"] = $this->environment->getCdFilial();
			$cdcontador = "APURACFATUR".$row["CDFILIAL"];
			$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
			$rowToSave = array(
			"CDFILIAL"    => $row["CDFILIAL"],
			"NRAPURACFAT" => $row['NRAPURACFAT'],
			"CDCLIENTE"   => $row['CDCLIENTE'],
			"DTINIAPURAC" => $row['DTINIAPUR'],
			"DTFIMAPURAC" => $row['DTFINAPUR'],
			"CDFILIAPUR"  => $row["CDFILIAPUR"],
			"CDTIPOFATUCT"=> '001',
			"__is_new"    => true
			);
		} else {
			if (!empty($ultimaApuracao) && $ultimaApuracao['IDBLOQUEADO'] == 'S') {
				$ultimaDtiniapur = \DateTime::createFromFormat('!d/m/Y', substr($ultimaApuracao["DTINIAPURAC"], 0, 10));
				$ultimaDtfimapur = \DateTime::createFromFormat('!d/m/Y', substr($ultimaApuracao["DTFIMAPURAC"], 0, 10));
				$dtiniapur = \DateTime::createFromFormat('!d/m/Y', substr($row["DTINIAPUR"], 0, 10));
				$dtfimapur = \DateTime::createFromFormat('!d/m/Y', substr($row["DTFINAPUR"], 0, 10));
				if (($dtiniapur >= $ultimaDtiniapur && $dtiniapur <= $ultimaDtfimapur) ||
				($dtfimapur >= $ultimaDtiniapur && $dtfimapur <= $ultimaDtfimapur)) {
					throw new \Exception("Já existe nota emitida para o período apurado.", 1);
				} else {
					$row["CDFILIAL"] = $this->environment->getCdFilial();
					$cdcontador = "APURACFATUR".$row["CDFILIAL"];
					$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
					$rowToSave = array(
						"CDFILIAL"    => $row["CDFILIAL"],
						"NRAPURACFAT" => $row['NRAPURACFAT'],
						"CDCLIENTE"   => $row['CDCLIENTE'],
						"DTINIAPURAC" => $row['DTINIAPUR'],
						"DTFIMAPURAC" => $row['DTFINAPUR'],
						"CDFILIAPUR"  => $row["CDFILIAPUR"],
						"CDTIPOFATUCT"=> '001',
						"__is_new"    => true
					);
				}
			} else {
				$row["CDFILIAL"] = $this->environment->getCdFilial();
				$cdcontador = "APURACFATUR".$row["CDFILIAL"];
				$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
				$rowToSave = array(
					"CDFILIAL"    => $row["CDFILIAL"],
					"NRAPURACFAT" => $row['NRAPURACFAT'],
					"CDCLIENTE"   => $row['CDCLIENTE'],
					"DTINIAPURAC" => $row['DTINIAPUR'],
					"DTFIMAPURAC" => $row['DTFINAPUR'],
					"CDFILIAPUR"  => $row["CDFILIAPUR"],
					"CDTIPOFATUCT"=> '001',
					"__is_new"    => true
				);
			}
		}
			
		return $rowToSave;
	}
	
	private function saveCtrlApurFat($rowsToSave) {

		$dataSet = new DataSet(null, $rowsToSave);
		$request = new DTO\Request\DataSet($dataSet, '', '', '');
		$rows = $this->crudService->getRowsToSave($request,'ctrlapurfat');
		$this->crudService->save('ctrlapurfat', $rows);
	}

	private function verificaNotaFiscal($row) {

		$conditions = array (
			array (
				"columnName" 	=> "CDFILIAL",
				"operator" 		=> "=",
				"value" 		=> $row["CDFILIAL"]
			),
			array (
				"columnName" 	=>"NRAPURACFAT",
				"operator" 		=>"=",
				"value" 		=> $row['NRAPURACFAT']
			),
			array (
          	  "columnName" 		=>'NRORG',
          	  "operator" 		=> "=",
          	  "value" 			=> $this->environment->getNrOrg()
          	)
		);

		$filterCriteria = new FilterCriteria('notafiscal', $conditions);
       	$filterCriteria->setDataSourceName('notafiscal');
       	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		return $dataSet->getRows();
	}
	private function getCdctrlnumenf($row){
		$conditions = array (
			array (
				"columnName" 	=>"CDFILIAL",
				"operator" 		=>"=",
				"value" 		=> $row["CDFILIAL"]
			),
			array (
         		"columnName" 	=>'NRORG',
         		"operator" 		=> "=",
         		"value" 		=> $this->environment->getNrOrg()
         	)
		);
		$filterCriteria = new FilterCriteria('fai00800_filial_serienf_ctrlnumenf', $conditions);
       	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
       	return $dataSet->getRows()[0]['CDCTRLNUMENF'];
	}
	private function verificaIntervalo($row, $cdctrlnumenf){

       	$conditions = array (
			array (
				"columnName" 	=>"CDCTRLNUMENF",
				"operator" 		=>"=",
				"value" 		=> $cdctrlnumenf
			),
			array (
				"columnName" 	=>"CDLOCALEMIS",
				"operator" 		=>"=",
				"value" 		=> $row['CDLOCAL']
			),
			array (
         		"columnName" 	=>'NRORG',
         		"operator" 		=> "=",
         		"value" 		=> $this->environment->getNrOrg()
         		)
		);
		$filterCriteria = new FilterCriteria('seqnumeloca', $conditions);
       	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
       	return $dataSet;
	}
	private function saveLocalEmissao($row, $dataSet, $cdctrlnumenf){
		$nrcdfrini = $dataSet->getRows()[0]['NRCDFRINI'];
        $nrcdfrfim = $dataSet->getRows()[0]['NRCDFRFIM'];
        $cdcontador = "CDFRLOCEMIS".$cdctrlnumenf.$row['CDLOCAL'];
        $nrsequencial = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
	    $conditions = array (
			array (
				"columnName" 	=> "CDFILIAL",
				"operator" 		=> "=",
				"value" 		=> $row['CDFILIAL']
			),
			array (
				"columnName" 	=> "NRAPURACFAT",
				"operator" 		=> "=",
				"value" 		=> $row['NRAPURACFAT']
			),
			array (
			  	"columnName" 	=> "NRORG",
			 	"operator" 		=> "=",
			 	"value" 		=> $this->environment->getNrOrg()
			)
		);
		$filterCriteria = new FilterCriteria('fai00800_ctrlapcdrf', $conditions);
		$this->parameterBagColumns = array ("CDFILIAL", "NRAPURACFAT", "NRORG");
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		if(empty($dataSet->getRows())){
			$conditions = array (
				array (
					"columnName" 	=>"CDCTRLNUMENF",
					"operator" 		=>"=",
					"value" 		=> $cdctrlnumenf
				),
				array (
					"columnName" 	=>"CDLOCALEMIS",
					"operator" 		=>"=",
					"value" 		=> $row['CDLOCAL']
				),
				array (
					"columnName" 	=>"DTCTRLAPCDFR",
					"operator" 		=>"=",
					"value" 		=> $row['DTINIAPUR']
				),
				array (
          		  	"columnName" 	=>'NRORG',
          		  	"operator" 		=> "=",
          		  	"value" 		=> $this->environment->getNrOrg()
          		)
			);
			$filterCriteria = new FilterCriteria('ctrlapcdrf', $conditions);
			$dataSet = $this->dataSourceManager->findBy($filterCriteria);
			if (!empty($dataSet->getRows())) {
				$rowToSave = array (
					array (
						"CDFILIALCENT"	=> $this->environment->getCdFilial(),
						"NRAPURACFATU"	=> $row['NRAPURACFAT'],
						"CDCTRLNUMENF"	=> $cdctrlnumenf,
						"CDLOCALEMIS" 	=> $row['CDLOCAL'],
						"DTCTRLAPCDFR"	=> $row['DTINIAPUR'],
						"__is_new"    	=> false
					)
				);
				$newDataSet = new DataSet(null, $rowToSave);
				$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
				$newRow = $this->crudService->getRowsToSave($newRequest,'ctrlapcdrf');
				$this->crudService->save('ctrlapcdrf', $newRow);
			} else {
				if($nrsequencial > $nrcdfrini && $nrsequencial < $nrcdfrfim){
					$rowToSave = array (
						array (
							"CDCTRLNUMENF"	=> $cdctrlnumenf,
							"CDLOCALEMIS" 	=> $row['CDLOCAL'],
							"CDNRCDFR" 		=> str_pad($nrsequencial, 20, "0", STR_PAD_LEFT),
							"CDFILIALCENT"	=> $this->environment->getCdFilial(),
							"NRAPURACFATU"	=> $row['NRAPURACFAT'],
							"IDPERIODFATU"	=> $row['IDPERIODFATC'],
							"IDSTATUSIMPR"	=> "N",
							"IDSITUCDFR"  	=> "P",
							"DTCTRLAPCDFR"	=> $row['DTINIAPUR'],
							"__is_new"    	=> true
						)
					);
					$newDataSet = new DataSet(null, $rowToSave);
					$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
					$newRow = $this->crudService->getRowsToSave($newRequest,'ctrlapcdrf');
					$this->crudService->save('ctrlapcdrf', $newRow);
					return true;
				} else {
					return false;
				}
			}
		}
	}

	public function realizaApura(DTO\Request $request, DTO\Response $response) {
		try {
			$rows = $request->getDataSet()->getRows();
			$responseDataSet = array ();
			$responseValida = $this-> validaApuracao($rows);
			if(!empty($responseValida)){
				 $response->addDataSet(new DataSet("invalidos", $responseValida));
			}
			else{
				foreach ($rows as $key => $row) {
					$rowToSave = $this->getNrApuracFatForRow($row);
					
					$this->entityManager->getConnection()->beginTransaction();
					
					//$this->saveCtrlApurFat($rowsToSave);
					
					$sql = "INSERT INTO CTRLAPURFAT
							(CDFILIAL, NRAPURACFAT, CDCLIENTE, DTINIAPURAC, DTFIMAPURAC, CDFILIAPUR, CDTIPOFATUCT)
							VALUES
							(:CDFILIAL, :NRAPURACFAT, :CDCLIENTE, :DTINIAPURAC, :DTFIMAPURAC, :CDFILIAPUR, :CDTIPOFATUCT)";
					
					$params = array (
						'CDFILIAL' => $rowToSave['CDFILIAL'],
						'NRAPURACFAT' => $rowToSave['NRAPURACFAT'],
						'CDCLIENTE' => $rowToSave['CDCLIENTE'],
						'DTINIAPURAC' => $rowToSave['DTINIAPURAC'],
						'DTFIMAPURAC' => $rowToSave['DTFIMAPURAC'],
						'CDFILIAPUR' => $rowToSave['CDFILIAPUR'],
						'CDTIPOFATUCT' => $rowToSave['CDTIPOFATUCT']
					);
					
					$prepare = $this->entityManager->getConnection()->prepare($sql);
					$prepare->execute($params);
					
					$params = array (
						'P_CDFILIAL' => $row['CDFILIAL'],
						'P_NRAPURACFAT' => $row['NRAPURACFAT']
					);
					$query = 'SELECT MAX(NRRETROFAT) NRRETROFAT
							FROM  CTRLAPURRETROFAT
							WHERE CDFILIAL    = :P_CDFILIAL AND NRAPURACFAT = :P_NRAPURACFAT';
					$sql = $this->entityManager->getConnection()->prepare($query);
					$sql->execute($params);
					$temRetroativo = $sql->fetchAll();
					array_push($responseDataSet, array ());
					array_push($responseDataSet[$key], $row["CDFILIAL"]);
					array_push($responseDataSet[$key], $row["NRAPURACFAT"]);
					array_push($responseDataSet[$key], $row['CDCLIENTE']);
					array_push($responseDataSet[$key], $row['NMRAZSOCCLIE']);
					if($temRetroativo[0]["NRRETROFAT"] == null){	
						
						if ($row["IDGERAPREVIAAPURAC"] == 'S') {
							$row["IDGERANOTA"] = 'N';
						} else {
							$row["IDGERANOTA"] = 'S';					
						}
						$row["GERAPREVIA"] = 'S';
						try{
						
							if ($row["IDCENTRALFATU"] === 'S') {
								$this->executaApuracaoGestaoDiretaCentral($row, $response, true);
							} else {
								$this->executaApuracaoGestaoDireta($row, $response, true);
							}
						}catch(\Exception $e){
							$this->entityManager->getConnection()->rollback();
							if(strrpos($e->getMessage(), 'EFETFAT_DIF_EFETPLA') === false){
								throw $e;	
							}
							else{
								array_push($responseDataSet[$key], $e->getMessage());
								continue;
							}
						}
						
						if ($row['UTILIZA_LOCAL']=='S') {
							
							if (empty($this->verificaNotaFiscal($row))) {
								array_push($responseDataSet[$key], "ERRO_SNF");
								$this->entityManager->getConnection()->rollback();
								
							} else {
								$cdctrlnumenf = $this->getCdctrlnumenf($row);
								$dataSet = $this->verificaIntervalo($row, $cdctrlnumenf);
								
								if (empty($dataSet->getRows()[0]['NRCDFRINI'])) {
									array_push($responseDataSet[$key], "ERRO_NCDRF");
									$this->entityManager->getConnection()->rollback();
									
								} else {
									if(!$this->saveLocalEmissao($row, $dataSet, $cdctrlnumenf)){
										array_push($responseDataSet[$key], "ERRO_CDRF");
										$this->entityManager->getConnection()->rollback();
									}
									else{
										$this->entityManager->getConnection()->commit();
									}
									
								}
							}
						}
						else{
							$this->entityManager->getConnection()->commit();
						}
					} else {
						array_push($responseDataSet[$key], "ERRO_RETROATIVO");
						$this->entityManager->getConnection()->rollback();
					}
				}
			
				$response->addDataSet(new DataSet("fai00800_realizaapura", $responseDataSet));
			}
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	public function gerarNF(DTO\Request $request, DTO\Response $response){
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$row = $request->getRow();
			$row["IDGERANOTA"] = 'S';
			$row["GERAPREVIA"] = 'N';
			if ($row["IDCENTRALFATU"] === 'S') {
				$this->executaApuracaoGestaoDiretaCentral($row, $response);
			} else {
				$this->executaApuracaoGestaoDireta($row, $response);
			}
			
			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function executaApuracaoGestaoDireta($row, $response, $throwError=false) {

		try {
			$procedure = new StoredProcedure($this->entityManager->getConnection(), 'APURAC_GESDIRETA');
			$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILIAL"],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row['CDCLIENTE'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row['DTINIAPUR'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $row['DTFINAPUR'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $row['NRAPURACFAT'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDGERANOTA', Param::PARAM_INPUT, $row['IDGERANOTA'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DTSAIDAPUR', Param::PARAM_INPUT, $row['DTSAIDAPUR'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, $row['TIPOOPER'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, NULL ,Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_GERAPREVIA', Param::PARAM_INPUT, $row['GERAPREVIA'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDFILIALCENTRAL', Param::PARAM_INPUT, $row['CDFILIAL'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRAPURACENTRAL', Param::PARAM_INPUT, $row['NRAPURACFAT'],Param::PARAM_TYPE_STR));
			$procedure->execute();
		} catch (\Exception $e) {
			if($throwError){
				throw $e;
			}
			else{
				$msg = $e->getMessage();
				if($e->getErrorCode() == 20501){
					$array_messages = explode("\n",$e->getMessage());
					$size = count($array_messages);
					if($size >= 3){
						$parts_message = explode("ORA-20501: ",$array_messages[2]);
						$size_parts = count($parts_message);
						if($size_parts >= 2){
							$msg = utf8_decode($parts_message[1]);
						}
					}
				}
				$response->setError(new Error($msg, 500));
			}
    	}
	}

	private function executaApuracaoGestaoDiretaCentral($row, $response, $throwError=false) {

		try {
			$procedure = new StoredProcedure($this->entityManager->getConnection(), 'APURAC_GESDIRETA_CENTRAL');
			$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILIAL"],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row['CDCLIENTE'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row['DTINIAPUR'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $row['DTFINAPUR'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $row['NRAPURACFAT'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDGERANOTA', Param::PARAM_INPUT, $row['IDGERANOTA'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DTSAIDAPUR', Param::PARAM_INPUT, $row['DTSAIDAPUR'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, $row['TIPOOPER'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_GERAPREVIA', Param::PARAM_INPUT, $row['GERAPREVIA'],Param::PARAM_TYPE_STR));
			$procedure->execute();
		} catch (\Exception $e) {
			if($throwError){
				throw $e;	
			}
			else{
				$response->setError(new Error($e->getMessage(), 500));
			}
    	}
	}

	public function salvaPreviaApurac(DTO\Request $request, DTO\Response $response) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$rows = $request->getDataSet()->getRows();
			$this->entityManager->getConnection()->setAutoCommit(false);
			foreach($rows as $row){
				$tabela = $row["TABELA"];
				$dataSourceName = strtolower("TRAB_APURAC_".$tabela);
				$tabelaRow = $this->getRowTabela($row, $tabela);
				
				if (isset($row["CHANGED_CLIENTE"]) && $row["CHANGED_CLIENTE"]) {
					$tabelaRow["CDGRSERVFAT"] = NULL;
				}
				
				$dataSet = new DataSet($dataSourceName, array($tabelaRow));
				$newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
				$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
				if (isset($row["CHANGED_CDTIPOOPER"]) && $row["CHANGED_CDTIPOOPER"]) {
					$this->getgrupoNFRows($row["CDFILIAL"], $row["NRAPURACENTRAL"], $row["CDCLIENTE"], $row["NRGRUPONF"], $row["CDTIPOOPER"], $row["NRSEQUENCIAL"]);
				}
				if (isset($row["CHANGED_NREFETIVOS"]) && $row["CHANGED_NREFETIVOS"]) {
					$rowComplemento = $this->complementaQuantidade($row, $rowsToSave[0], $tabela, $dataSourceName);
					array_push($rowsToSave, $rowComplemento);
				}
				$this->crudService->save($dataSourceName, $rowsToSave);
				$this->entityManager->getConnection()->commit();
			}
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function getgrupoNFRows($cdfilial, $nrapuracfat, $cdcliente, $nrgruponf, $cdtipooper, $nrsequencial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			),
			array(
				"columnName" => "CDCLIENTE",
				"operator" => "=",
				"value" => $cdcliente
			),
			array(
				"columnName" => "NRGRUPONF",
				"operator" => "=",
				"value" => $nrgruponf
			),
			array(
				"columnName" => "NRSEQUENCIAL",
				"operator" => "<>",
				"value" => $nrsequencial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai00800_trab_apurac", $conditions);
		$rows = $dataSet->getRows();
		foreach ($rows as $row) {
			$tabela = $row["TABELA"];
			$row["CDTIPOOPER"] = $cdtipooper;
			$rowTabela = $this->getRowTabela($row, $tabela);
			$dataSourceName = strtolower("TRAB_APURAC_".$tabela);
			$dataSet = new DataSet($dataSourceName, array($rowTabela));
			$newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
			$this->crudService->save($dataSourceName, $rowsToSave);
		}
	}

	private function getRowTabela($row, $tabela) {
		$changedRow = array (
			"NRSEQUENCIAL"      	   => $row["NRSEQUENCIAL"],
			"CDFILIALAP".$tabela       => $row["CDFILIAL"],
			"CDCLIENTEAP".$tabela      => $row["CDCLIENTE"],
			"CDSERVFILIAP".$tabela     => $row["CDSERVFILI"],
			"VRPRECUNITAP".$tabela     => $row["VRPRECOUNIT"],
			"CDPRODUTOAP".$tabela      => $row["CDPRODUTO"],
			"NREFETIVOSAP".$tabela     => $row["NREFETIVOS"],
			"NRAPURAC".$tabela         => $row["NRAPURAC"],
			"DTAPURACFAT".$tabela      => $row["DTAPURACFAT"],
			"NMSERVICO".$tabela        => $row["NMSERVICO"],
			"CDGRSERVFAT"              => $row["CDGRSERVFAT"],
			"CDTIPOOPER"               => $row["CDTIPOOPER"],
			"VRDESCONTO"               => $row["VRDESCONTO"],
			"DSJUSTIFICATIVA"          => $row["DSJUSTIFICATIVA"],
			"NRGRUPONF"                => $row["NRGRUPONF"],
			"NRPEDIDO"                 => $row["NRPEDIDO"],
			"NRITEMPED"                => $row["NRITEMPED"],
			"CDFILIAGREGADA"           => $row["CDFILIAGREGADA"],
			"NRAPURACENTRAL"           => $row["NRAPURACENTRAL"],
			"__is_new"                 => false
		);
		if ($tabela == "TU") {
			$changedRow["CDTURNOTU"] = $row["CDTURNO"];
		}
		return $changedRow;
	}

	private function complementaQuantidade($row, $newRow, $tabela, $dataSourceName) {
		$newRow["NREFETIVOSAP".$tabela] = floatval($row["CHANGED_NREFETIVOS"]) - floatval($row["NREFETIVOS"]);
		$newRow["VRDESCONTO"] = 0;
		$newRow["DSJUSTIFICATIVA"] = null;
		$newRow["NRPEDIDO"] = null;
		$newRow["NRITEMPED"] = null;
		$newRow["IDTABELA"] = $tabela;
		$newRow["NRSEQUENCIAL"] = $this->getMaxNrSequencial($row["NRAPURACENTRAL"], $row["CDFILIAL"]);
		$newRow["__is_new"]= true;
		return $newRow;
	}

	private function getMaxNrSequencial($nrapuracfat, $cdfilial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai00800_trabapuracmaxnrseq", $conditions);
		$rows = $dataSet->getRows();
		if (!empty($rows)) {
			return $rows[0]["NRSEQUENCIAL"] + 1;
		} else {
			return 1;
		}
	}

	public function agruparItens(DTO\Request $request, DTO\Response $response) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$rows = $request->getDataSet()->getRows();
			if (count($rows) == 1) {
				$this->validaItensGrupo($rows[0]["NRAPURACENTRAL"], $rows[0]["CDFILIAL"], $rows[0]["NRGRUPONF"], $rows[0]["NRSEQUENCIAL"]);
			}
			$tabela = $rows[0]["TABELA"];
			$dataSourceName = strtolower("TRAB_APURAC_".$tabela);
			$nrgruponf = $this->getMaxNrGrupoNF($rows[0]["NRAPURACENTRAL"], $rows[0]["CDFILIAL"]);
			foreach($rows as &$row) {
				$row["NRGRUPONF"] = $nrgruponf;
				$newRequest = new DTO\Request\DataSet(new DataSet(null, array($row)), "", "", "");
				$this->salvaPreviaApurac($newRequest, $response);
			}
			$dataSet = new DataSet("fai00800_agruparitens", array(array("NRGRUPONF" => $nrgruponf)));
			$response->addDataSet($dataSet);
			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function validaItensGrupo($nrapuracfat, $cdfilial, $nrgruponf, $nrsequencial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			),
			array(
				"columnName" => "NRGRUPONF",
				"operator" => "=",
				"value" => $nrgruponf
			),
			array(
				"columnName" => "NRSEQUENCIAL",
				"operator" => "<>",
				"value" => $nrsequencial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai00800_trab_apurac", $conditions);
		$rows = $dataSet->getRows();
		if (empty($rows)) {
			throw new \Exception("unicoNoGrupo");
		}
	}

	private function getAllTrabApurac($nrapuracfat, $cdfilial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai00800_trab_apurac", $conditions);
		$rows = $dataSet->getRows();
	}
	private function getMaxNrGrupoNF($nrapuracfat, $cdfilial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai00800_trabapuracmaxnrgruponf", $conditions);
		$rows = $dataSet->getRows();
		if (!empty($rows)) {
			return $rows[0]["NRGRUPONF"] + 1;
		} else {
			return 1;
		}
	}

	public function deleteDadosAdic(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$cdfilial = $row["CDFILIAL"];
		$nrapuracfat = $row["NRAPURACFAT"];
		$sql = "DELETE FROM DADOSADIC_APURAC WHERE CDFILIAL = :CDFILIAL AND NRAPURACFAT = :NRAPURACFAT AND NRORG = :NRORG";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRAPURACFAT" => $nrapuracfat,
            "NRORG"      => $this->environment->getNrOrg()
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
    	$query->execute($params);
	}
	
	private function validaApuracao($rows){
		$responseData = array ();
			foreach ($rows as $key => $row) {
				$params = array (
					'P_CDFILIAPUR' => $row['CDFILIAPUR'],
					'P_CDCLIENTE' => $row['CDCLIENTE'],
					'DTINIAPURAC' => $row['DTINIAPUR'],
					'DTFIMAPURAC' => $row['DTFINAPUR']
				);
				$query = 'SELECT C.CDCLIENTE, DTINIAPURAC, DTFIMAPURAC
						  FROM CTRLAPURFAT C
						   WHERE C.CDFILIAPUR = :P_CDFILIAPUR
						   AND C.CDCLIENTE = :P_CDCLIENTE
						   AND C.CDTIPOFATUCT = 001
						   AND ((:DTINIAPURAC BETWEEN DTINIAPURAC AND DTFIMAPURAC) OR
						        (:DTFIMAPURAC BETWEEN DTINIAPURAC AND DTFIMAPURAC) OR
						        (DTINIAPURAC BETWEEN :DTINIAPURAC AND :DTFIMAPURAC) OR
						        (DTFIMAPURAC BETWEEN :DTINIAPURAC AND :DTFIMAPURAC)
						       )';
				$sql = $this->entityManager->getConnection()->prepare($query);
				$sql->execute($params);
				$temRetroativo = $sql->fetchAll();
				if(!empty($temRetroativo)){
					array_push($responseData, $temRetroativo);
					

				}
			}
		return $responseData;
	}
	
	public function apuracaoAindaRealizada(DTO\Request $request, DTO\Response $response){
		try{
			$row = $request->getRow();
			$connection = $this->entityManager->getConnection();
		    $sql = "SELECT C.CDFILIAL
					  FROM CTRLAPURFAT C
					 WHERE C.CDFILIAL = :CDFILIAL
					   AND C.NRAPURACFAT = :NRAPURACFAT";
			
			$params = array (
						'CDFILIAL' => $row['CDFILIAL'],
						'NRAPURACFAT' => $row['NRAPURACFAT']
					);
			$prepared = $connection->prepare($sql);
			$prepared->execute($params);
			$resultado = $prepared->fetchAll();
			$resultado = (count($resultado) > 0) ? [true] : [false];
			$response->addDataSet(new DataSet("existe", $resultado));
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
}