<?php
namespace Teknisa\FAI\Modules\Parametros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;

class Fai01100ParametrosTipoFaturamento extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = 'fai01100_filial_tipo_faturamento';

    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;
    protected $util;
    protected $newCode;
    protected $connection;
    
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $util, $newCode, $connection) {
        parent::__construct($crudService, $parameterBag);
        $this->crudService = $crudService;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->util = $util;
        $this->newCode = $newCode;
        $this->connection = $connection;
    }
    
    public function loadFiliais(DTO\Request\DataSet $request, DTO\Response $response) {
        $data = $request->getDataSet()->getRows();
        $resp = array();
        foreach($data as $row) {
            $conditions = array(
                array(
                    "columnName" => "CDTIPOFATUR",
                    "operator"   => "=",
                    "value"      => $row['CD']
                )
            );
            $filterCriteria = new FilterCriteria("fai01100_filial_tipo_faturamento", $conditions);
            $newFilter = $this->crudService->populateParameterBag($filterCriteria, array("CDTIPOFATUR"));
            $result = $this->dataSourceManager->findBy($newFilter);
            $filiais = $result->getRows();
            foreach($filiais as $filial) {
                $filial['PARENT'] = $row['ID'];
                $filial['ID'] = $filial['CDFILIAL'];
                $filial['NAME'] = $filial['CDFILIAL'] . ' | ' .$filial['NMFILIAL'];
                $filial['WINDOW'] = strtolower($row['ID']).'FilialWidget';
                $resp[] = $filial;
            }
        }
        $dataSet = new Dataset('filiaisTipoFat', $resp);
        $response->addDataSet($dataSet, 'filiaisTipoFat');
    }
    
    public function saveOperacao(DTO\Request\DataSet $request, DTO\Response $response){
        $this->dataSourceName = "tipooperdevolfat";
        $row = $request->getDataSet()->getRows();
        $error = false;
        
        try{
            $this->connection->beginTransaction();
            // Verifica se já possui dados no banco.
            foreach ($row as $data){
                $validDuplicate = 'SELECT * FROM TIPOOPERDEVOLFAT WHERE CDTIPOOPER = :CDTIPOOPER';
                $params = array(
                    'CDTIPOOPER'  => $data['CDTIPOOPER']
                );
                $duplicate = $this->entityManager->getConnection()->fetchAll($validDuplicate, $params);
                if(!empty($duplicate)){
                    $error = true;
                    $response->addDataSet(new DataSet ('validDuplicate', array(true)));
                }
            }
            
            if(!$error){
                foreach($row as $data){
                    $data['NRTIPOOPERDEVOLFAT'] = $this->newCode->getCodeByTableName('TIPOOPERDEVOLFAT', $this->environment->getNrorgTrab(), 1, 10, true);
                }
                
                parent::save($request, $response);
                $this->connection->commit();
                $response->addDataSet(new DataSet ('saveOperacao', array(true)));
            }
        }catch(\Exception $e){                
            $this->connection->rollback();
            throw $e;
        }
    }
    
    private function getAutPerfilAcesso(){
        $dataSourceName = 'autperfilacesso';
        $result = $this->crudService->getFilteredDataSet($dataSourceName, array())->getRows();
        return $result;
    }
    
    private function insertAutPerfilAcesso($connection, $row){
        $sql = "INSERT INTO AUTPERFILACESSO (
	                NRAUTPERFACESSO, 
	                NRPERFILACESSO, 
	                IDAUTPERFACESSO, 
	                DSAUTPERFACESSO, 
	                NRORG, 
	                DTINCLUSAO, 
	                DTULTATU, 
	                NRORGINCLUSAO, 
	                CDOPERINCLUSAO, 
	                NRORGULTATU, 
	                CDOPERULTATU
	           ) VALUES (
	                :NRAUTPERFACESSO,
                    :NRPERFILACESSO,
                    :IDAUTPERFACESSO,
                    :DSAUTPERFACESSO,
                    :NRORG,
                    SYSDATE,
                    SYSDATE,
                    :NRORGINCLUSAO,
                    :CDOPERINCLUSAO,
                    :NRORGULTATU,
                    :CDOPERULTATU
                    )";
	   $params = array(
            "NRAUTPERFACESSO" => $row['NRAUTPERFACESSO'],
            "NRPERFILACESSO"  => $row['NRPERFILACESSO'],
            "IDAUTPERFACESSO" => $row['IDAUTPERFACESSO'],
            "DSAUTPERFACESSO" => $row['DSAUTPERFACESSO'],
            "NRORG"           => $row['NRORG'],
            "NRORGINCLUSAO"   => $row['NRORGINCLUSAO'],
            "CDOPERINCLUSAO"  => $row['CDOPERINCLUSAO'],
            "NRORGULTATU"     => $row['NRORGULTATU'],
            "CDOPERULTATU"    => $row['CDOPERULTATU']
            );
	   $stmt = $connection->prepare($sql);
	   $stmt->execute($params);
    }
    
    public function deleteAutPerfilAcesso($connection, $row){
        $sql = "DELETE FROM AUTPERFILACESSO WHERE IDAUTPERFACESSO = :IDAUTPERFACESSO AND NRORG = :NRORG";
        $params = array(
            "IDAUTPERFACESSO" => $row['IDAUTPERFACESSO'],
            "NRORG" => $row['NRORG']
        );
	   $stmt = $connection->prepare($sql);
	   $stmt->execute($params);
    }
    
    public function updateAutPerfilAcesso($connection, $row){
        $sql = "UPDATE AUTPERFILACESSO SET NRPERFILACESSO = :NRPERFILACESSO WHERE IDAUTPERFACESSO = :IDAUTPERFACESSO AND NRORG = :NRORG";
        $params = array(
            "NRPERFILACESSO"  => $row["NRPERFILACESSO"],
            "IDAUTPERFACESSO" => $row["IDAUTPERFACESSO"],
            "NRORG" => $row["NRORG"]
        );
        $stmt = $connection->prepare($sql);
        $stmt->execute($params);
	}
    
    public function saveGrupoOperadoresAutPerfilAcesso(DTO\Request\Row $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $autPerfilAcesso = $this->getAutPerfilAcesso();
            $permissions = $row['permissions'];
            $currentRow = $row['currentRow'];
            $orderPermissionsSelect = array_column($autPerfilAcesso, 'IDAUTPERFACESSO');
            $basicRow = array(
                'CDOPERINCLUSAO' => $this->environment->getCdOperador(),
                'NRORGINCLUSAO' => $this->environment->getNrOrg(),
                'CDOPERULTATU' => $this->environment->getCdOperador(),
                'NRORGULTATU' => $this->environment->getNrOrg(),
                'NRORG' => $this->environment->getNrOrg()
            );
            $connection = $this->entityManager->getConnection();
            foreach($permissions as $namePermission => $information){
              $index = array_search($namePermission, $orderPermissionsSelect);
              $connection->beginTransaction();
              try{
                  if($index === false && isset($currentRow[$information['value']]) && $currentRow[$information['value']]){
                      $rowToInsert = $basicRow;
                      $rowToInsert['NRAUTPERFACESSO'] = $this->newCode->getCodeByTableName('AUTPERFILACESSO'.$rowToInsert['NRORG'], $rowToInsert['NRORG']);
                      $rowToInsert['NRPERFILACESSO'] = $currentRow[$information['value']];
                      $rowToInsert['IDAUTPERFACESSO'] = $namePermission;
                      $rowToInsert['DSAUTPERFACESSO'] = $information['label'];
                      $this->insertAutPerfilAcesso($connection, $rowToInsert);
                      $response->addDataSet(new DataSet("InsertAutPerfilAcesso", $rowToInsert));
                  } else if(!isset($currentRow[$information['value']]) || (isset($currentRow[$information['value']]) && !$currentRow['DSGRUPOPER'])){
                      $rowToDelete = $basicRow;
                      $rowToDelete['IDAUTPERFACESSO'] = $namePermission;
                      $this->deleteAutPerfilAcesso($connection, $rowToDelete);
                      $response->addDataSet(new DataSet("deleteAutPerfilAcesso", $rowToDelete));
                  } else if(isset($currentRow[$information['value']]) && $currentRow[$information['value']] !==  $autPerfilAcesso[$index]['NRPERFILACESSO']){
                      $rowToUpdate = $basicRow;
                      $rowToUpdate["NRPERFILACESSO"] = $currentRow[$information['value']];
                      $rowToUpdate['IDAUTPERFACESSO'] = $namePermission;
                      $this->updateAutPerfilAcesso($connection, $rowToUpdate);
                      $response->addDataSet(new DataSet("updateAutPerfilAcesso", $rowToUpdate));
    	            }
                  $connection->commit();
              } catch(\Exception $e){
                  $connection->rollback();
                  $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
              }
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}