<?php
namespace Teknisa\FAI\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DependencyInjection\InstanceManager;

use Teknisa\Libs\Util\SMTP;

use Teknisa\FAI\Util\Utilities;

class SMTPUnit {
	
	private $conn;
	private $env;
	private $pathToSave = '/tmp/';
	private $temporaryAttachments = [];
	private $smtpParams = [];
	private $subject = '';

	public function __construct($connection, $environment, $smtpParams = []){
		$this->conn = $connection;
		$this->env = $environment;
		if(empty($smtpParams)){
			$this->smtpParams = InstanceManager::getInstance()->getParameter('smtp_params');
		} else {
			$this->smtpParams = $smtpParams;
		}
	}

	public function findSmtpParamsUnit( $cdfilial ){
		$sql = <<<SQL
			SELECT
				NVL(DSLOGINSMTP, DSEMAILFILI) AS USERNAME,
				DSSMTPEMAILF   AS HOST,
				CDSENHEMAILF   AS PASSWORD,
				CASE
					WHEN IDUTTLSEMAIL = 'S' THEN
						'tls'
					WHEN IDUTSSLEMAIL = 'S' THEN
						'ssl'
					ELSE
						''
				END AS SMTPSECURE,
				NVL(DSEMAILFILI, DSLOGINSMTP) AS EMAIL_FROM,
				CASE
					WHEN NVL(DSLOGINSMTP, DSEMAILFILI) IS NOT NULL
						AND CDSENHEMAILF IS NOT NULL THEN
						'S'
					ELSE
						'N'
				END AS AUTH
			FROM
				ENDEFILI
			WHERE
				CDFILIAL = :P_CDFILIAL
				AND IDTPENDEFILI = 'P'
				AND DSSMTPEMAILF IS NOT NULL
SQL;
		$stmt = $this->conn->prepare($sql);
		$stmt->execute([
			'P_CDFILIAL' => $cdfilial
		]);
		$result = $stmt->fetch();
		$params = [];
		if(!empty($result)){
			$params = [
				'host'		 => $result['HOST'],
				'username'	 => $result['USERNAME'],
				'password'	 => $result['PASSWORD'],
				'smtpSecure' => $result['SMTPSECURE'],
				'from'		 => $result['EMAIL_FROM'],
				'smtpAuth' 	 => $result['AUTH']
			];
			$hostAndPort = Utilities::getSmtpSettingsFromAddress($params['host']);
			$params['host'] = $hostAndPort['host'];
			$params['port'] = $hostAndPort['port'] ?? '';
			$params['password'] = Utilities::getDelphiPasswordEncrypted($params['password'], false);
		}
		return $params;
	}

	private function timeStamp(){
		return (new \DateTime())->getTimestamp();
	}

	public function findEmailUnit(DTO\Request\Row $request, DTO\Response $response){
		$sql = <<<SQL
		SELECT
			DSEMAILFILI AS EMAIL_FROM
		FROM
			ENDEFILI
		WHERE
			( CDFILIAL = :P_CDFILIAL )
			AND IDTPENDEFILI = 'P'
			AND NRORG = :P_NRORG
SQL;
		$row = $request->getRow();
		$cdfilial = $row['CDFILIAL'] ?? null;
		$stmt = $this->conn->prepare($sql);
		$stmt->execute([
			'P_CDFILIAL' => $cdfilial,
			'P_NRORG'    => $this->env->getNrOrg()
		]);
		$result = $stmt->fetch();
		if($result === false) $result = ["EMAIL_FROM" => null];
		$response->addDataSet( new DataSet("result", $result) );
	}

	public function addReportToAttachment($reportUrl, $nameFile, $extension = 'pdf') {
		$reportUrl = $reportUrl;
		$pathToSave = $pathToSave ?? $this->pathToSave;
		$time = $this->timeStamp();
		$filePath = $pathToSave."/{$nameFile}_$time.$extension";
		file_put_contents($filePath, fopen($reportUrl, 'r'));
		$this->temporaryAttachments[$filePath] = "$nameFile.$extension";
	}

	public function deleteTemporaryFiles(){
		foreach ( $this->temporaryAttachments as $fileToDelete => $mailFile ) {
			if(realpath($fileToDelete)) unlink($fileToDelete);
		}
	}

	public function sendMail($message, $address, $addressCC = '', $addressBCC = '', $addressReplyTo = ''){
		$smtp = new SMTP($this->smtpParams);
		if(!empty($this->temporaryAttachments)){
			foreach($this->temporaryAttachments as $fileToAdd => $mailFile) {
				$smtp->addAttachment($fileToAdd, $mailFile);
			}
		}
		if(!empty($this->subject)){
			$smtp->setSubject($this->subject);
		}
		$smtp->sendMail($message, $address, $addressCC, $addressBCC, $addressReplyTo);
	}

	public function setSubject( $subject ){
		$this->subject = $subject;
	}

	public function setPathToSave( $pathToSave ) {
		$this->pathToSave = $pathToSave;
	}

	public function getSubject(){
		return $this->subject;
	}

	public function getPathToSave() {
		return $this->pathToSave;
	}

	public function setSmtpParams($smtpParams) {
		$this->smtpParams = $smtpParams;
	}

	public function getSmtpParams(){
		return $this->smtpParams;
	}
}